#!/usr/bin/perl 
#use strict;

use Bio::TreeIO;


# This function will read in a treefile, and the corresponding alignment file,
# It will then split into subsets of sequences depending on $max

# It creates a "guidetree.dnd" file, which shows the order the files should be realigned in.

# WARNING: have to remove all seq files
system("rm *.seq\n");

#inputs
my $treefile=$ARGV[0];
my $seqfile=$ARGV[1];
my $numberofarguments =@ARGV;
my $max;
if($numberofarguments==3){
	$max=$ARGV[2];
	print "Max number of sequences per file is $max\n";
}else { 
	$max=0;
}






#Read in the tree 
my $treeio = new Bio::TreeIO( -file => $treefile) ;
my $tree = $treeio->next_tree or die "No Tree\n"; 
 
my %data;

#Calculates number of leaves for each ancestor.... 
my $total;
for my $node ( $tree->get_leaf_nodes ) {
    &inc_ancestor($node,\%data);  # Note the way the function is declared...
    $total++;
    my $leafid=$node->internal_id;
    $data{$leafid}=1;
}
print "Total number of leaves $total\n";


# Now split all leaves based on the $max 

if($max==0){ 
	$max=($total/2)+1;
        print "setting max number of sequences to $max\n";
	}
my $count=0;
my $seqset=0;

# This is a sort based on the VALUES i.e. the number of leaves....
my @sorted = sort { $data{$b} <=> $data{$a}} keys %data;
my $i=0;
my $size = @sorted;
while($i<$size){
    my $id=$sorted[$i];
    if ($data{$id}<=$max){
  #  print "ID: $id Number of leaves: " ,$data{$id},"\n"; 
    for my $node ( $tree->find_node(-internal_id =>$id)){
   # for my $node ( $tree->get_nodes){
	if(($id == $node->internal_id) && ($data{$id}<=$max) && (0==$node->has_tag(1))){
        	$node->add_tag_value(1,"done");
                my $mycount=0;
                my @leaves; 
        	if($node->is_Leaf){
               	my $leafid = $node->id;
               	$count++;
                $mycount++;
		#print "Adding $leafid to $id Count: $count\n"; 
                push @leaves,$leafid;

       		}
                else{ 
        		for my $newnode ( $node->get_all_Descendents){
        			$newnode->add_tag_value(1,"done");
       
        			if($newnode->is_Leaf){
                		my $leafid = $newnode->id;
                		$count++;
                                $mycount++;
				#print "Adding $leafid to $id Count: $count\n"; 
                                push @leaves,$leafid;
       				}
			}
   		}
  #               print "Adding $mycount to ID:$id Count so far is $count\n";

                $seqset++; # File counter....
                assignleaves(\@leaves,\$seqset,\$seqfile);               
        # Seqset will be the file name, and $seqset.seq

           my $seqsetfile=$seqset.".seq"; # Appending an extension
           $node->add_tag_value(2,$seqsetfile);
           $node->id($seqsetfile);
           $node->remove_all_Descendents();

    	}
 
     }
}
$i++;
        #If we have already assigned all the leaves, exit out of loop..
	if($count==$total){
		print "have assigned all leaves....exiting\n";
                print "There are  $seqset sequence files\n";
		$i=$size+2;
	}
}



# Check to make sure all the leaves have been allocated...
if ($count!=$total){
die "Error:There was a problem assigning leaves\n";
}


# Write the guide tree out
my $treeout=Bio::TreeIO->new(-file=>">guidetree.dnd") or die "Couldn't create output tree file\n";
$treeout->write_tree($tree);



# This is a recursive function, which will calculate the number of 
sub inc_ancestor {
    my $node = shift;
    my $data = shift;
    return unless defined $node && defined $node->ancestor;
    my $id = $node->ancestor->internal_id;
    $data->{$id}++; # This increments the count for the node..
    &inc_ancestor($node->ancestor,$data);
}



# This will assign the leaves to a sequence file....
# starting from an alignment file....
sub assignleaves(){

use Bio::AlignIO;
use Bio::SeqIO;

my ($leaves,$seqset,$seqfile)=@_;

chomp($$seqfile);
my $seqsetfile=">".$$seqset.".seq"; # Appending an extension

my $alignfile=$$seqfile;
my $alignio = new Bio::AlignIO(
                               -file => $alignfile) or die;

my $output = Bio::SeqIO->newFh(-file=> $seqsetfile, '-format' => 'fasta');

my $aln = $alignio->next_aln;


my $numberofleaves= @{$leaves};
my @sortedleaves= sort {$a cmp $b } @{$leaves};

for(my $j=0;$j<$numberofleaves;$j++){
	my $found = 0 ;
        # New look for each leave in the alignment file  
        foreach my $seq($aln->each_seq){
        	if ($sortedleaves[$j] eq ($seq->id)){
                
#        	print $seq->id, " should be added\n";
               
            #This removes the gaps from the aligned sequence. 
                my $newseqid = $seq->id;
                my $newseqseq = $seq->seq;
                $newseqseq =~s/-//g;

                my $newseq=Bio::Seq->new(-seq => $newseqseq, -id => $newseqid);

		print $output $newseq;  
                $found=1;
        	}
	}
        
	if ($found == 0){
		die "ERROR: have not found $sortedleaves[$j] Is this the correct alignment file $alignfile\n";
	}

}


}


