

## Uses "guidetree.dnd" to align sequences found in "guidetree.dnd"
## Guidetree created by treebasedsplitting.pl
## Uses Muscle to align profiles, SAGA to score alignment
## Uses ./convertaligntoseq-muscle.pl to create fastafiles for muscle from alignment
## Uses ./convertseqtoalign-muscle.pl to create clustalw alignment from fastafiles
## Uses ./bestfirst-muscle.pl to improve alignment
## Uses ./removenumbers.pl to remove extra numbers from alignment

#!/usr/bin/perl -w
use strict;

                                                                                
use Bio::TreeIO;
use Bio::SeqIO;
use Bio::AlignIO;

my $treefile = "guidetree.dnd";
my $treeio = new Bio::TreeIO( -file => $treefile) ;
my $tree = $treeio->next_tree or die "No Tree\n";


my @currentlist;
my @nextlist;

for my $node ( $tree->get_leaf_nodes ) {

	$node->add_tag_value(1,"done");
	
 	#Now want to run the method....
	my $seqfile = $node->id;
	#have to open the file and make sure that it has more than two sequences....

	my $seqs = Bio::SeqIO->new(-file=> $seqfile, '-format' => 'fasta') or die "Could not open sequence file: $seqfile\n";
        
	my $count=0;
        while (my $seq = $seqs->next_seq()){
	 $count++;
	}
        if($count>1){
        my $outfile=$seqfile.".aln";
        system("t_coffee $seqfile -outfile=$outfile -quiet=stdout > output");
        system("rm $seqfile");
        $node->id($outfile);
	}
	if($count==1){ }
	if($count==0){ die "!!no sequences : $seqfile!!\n";}
       
	 my $parent = $node->ancestor();
        
	if (defined($parent)){
        	my $parentid=$parent->internal_id;
       	 	push @currentlist,$parentid;        
	}
	

}



@currentlist=removeduplicates(@currentlist);

my $size = @currentlist;
my $i=0;
my $rootnode = $tree->get_root_node();

while($i<$size){
        print "processing\n";	
	for my $node ($tree->find_node(-internal_id => $currentlist[$i])){

	if(!($node->has_tag(1))){
	# Now have to check each descendent to make sure both have been done
		my $descendentcheck = 0 ;
                my $count=0;
                my @descendentid;
		for my $descendentnode ( $node->each_Descendent()){
                        
			$descendentcheck=$descendentcheck+$descendentnode->has_tag(1);
                        $descendentid[$count]=$descendentnode->id;
                        $count++;
		
		}
                
                if($count!=2){
		die "Not rooted? : more than two descendents\n";
		}

		if($descendentcheck==$count){

			$node->add_tag_value(1,"done");
                        $node->id($node->internal_id);
                        my $outfile=$node->id;
			$outfile=$outfile.".aln";
                        my $outfiletmp = "output";
                        
			$node->id($outfile);
	#		print "Good to go:have found it ",$node->internal_id , "\n";
                        my $profile1=$descendentid[0];
			my $profile2=$descendentid[1];    
                       
                        print "$profile1 and $profile2 \n"; 
                        # Have to conver the profiles into fasta format....                        
                        if ($profile1 =~/\.aln/){
			system("./convertaligntoseq-muscle.pl $profile1 profile1.fasta");
			} else { system("cp $profile1 profile1.fasta");}
                        if ($profile2 =~/\.aln/){
			system("./convertaligntoseq-muscle.pl $profile2 profile2.fasta");
			}else { system("cp $profile2 profile2.fasta");}
                       
			system("muscle33 -profile -in1 profile1.fasta -in2 profile2.fasta -out muscle.fasta\n");
                        
			system("./convertseqtoalign-muscle.pl muscle.fasta tmp.aln");
                        system("mv tmp.aln $outfile");
		#	system("clustalw -profile1=$profile1 -profile2=$profile2 -outfile=$outfile > $outfiletmp\n");
                        system("rm $profile1 $profile2 muscle.fasta profile1.fasta profile2.fasta \n");

                        my $newalignment =new Bio::AlignIO(
                                		-file => $outfile) or die "Died Could not open $outfile\n";
                        my $aln = $newalignment->next_aln or die "not an alignment file $outfile\n";
                        my $numberofseq=$aln->no_sequences;

                        if ($numberofseq>2){
		#	if($node->internal_id == $rootnode->internal_id){
				print "Will try and improve..\n";
				#my $currentscore =  AlignmentScore($outfiletmp); 
				my $currentscore =  -9999999; 
                        	#print "Current Score is $currentscore\n";
                                        if (defined($currentscore)) {
					if ($currentscore>-9999999999){
						system("./bestfirst-muscle.pl $outfile \n");
                                               system("./removenumbers.pl best.aln");
                                                #have to check that best.aln is a valid alignment file....
                        my $newalignment =new Bio::AlignIO(
                                		-file => 'best.aln') or die "Died Could not open $outfile\n";
                        my $aln = $newalignment->next_aln or die "not an alignment file $outfile\n";
                        my $numberofseq=$aln->no_sequences;
                                                if ($numberofseq>0){
							system("cp best.aln $outfile");
						}
					}
					}
                                        else {
					print "error: $currentscore is blank.....check output file\n";
					die;
					}
		#	}
			}

                        my $parent = $node->ancestor();
                        if (defined($parent)){
                        my $parentid=$parent->internal_id;
        		push @nextlist,$parentid;        
			}

		} else{
			my $nodeid = $node->internal_id;
                        push @nextlist,$nodeid;

		}
			
	
	}
}

$i++;
        if($i==$size){
                my $tmp=@nextlist;
        	#if(defined(@nextlist)){
        		if($tmp>0){
				@nextlist=removeduplicates(@nextlist);
        			@currentlist=@nextlist;
        			$size=@currentlist;
        			if($size>0){$i=0;}
#        				print "Processing next phase\n";
        				undef @nextlist;
				}
		#	}
	}

}




$rootnode = $tree->get_root_node();
my $outfile = $rootnode->id();
system ("cp $outfile combined.aln");

print "\nperforming sanity check\n";
for my $node ($tree->get_nodes){
	if(!($node->has_tag(1))){
        my $nodeid = $node->internal_id;
	print "Didn't work for $nodeid\n";
	}
}	




sub removeduplicates(){

my @array = @_;
my @sortedarray=sort {$a <=> $b } @array;

my $size = @sortedarray;
my @tmp;
my $i=1;

push @tmp,$sortedarray[0];

while($i<$size){

if (!($sortedarray[$i]==$sortedarray[$i-1])){
	push @tmp,$sortedarray[$i];
	}
$i++;
}

@tmp;
}


sub AlignmentScore(){
#!/usr/bin/perl
my   $CurrentScore1;
                                                                                                                                                             
my $file=$_[0];
open(RESULTS,$file) or die;
                                                                                                                                                             
while(<RESULTS>){
    if(/Alignment Score ([0123456789\-]+)/){
    $CurrentScore1=$1;
        }
}
                                                                                                                                                             
close(RESULTS);

$CurrentScore1;

}

